@@ -14,7 +14,7 @@ implementation (like GTK, Qt, or CLI).
 # the full text of the license.
 
 import glob, sys, os.path, optparse, traceback, locale, gettext, re, io
-import errno, zlib
+import errno, zlib, gzip
 import subprocess, threading, webbrowser
 import signal
 import time
@@ -33,7 +33,7 @@ else:
     from configparser import ConfigParser
     PY3 = True
 
-__version__ = '2.20.9'
+__version__ = '2.20.11'
 
 
 def excstr(exception):
@@ -66,19 +66,6 @@ def still_running(pid):
     return True
 
 
-def find_snap(package):
-    import requests_unixsocket
-
-    session = requests_unixsocket.Session()
-    try:
-        r = session.get('http+unix://%2Frun%2Fsnapd.socket/v2/snaps/{}'.format(package))
-        if r.status_code == 200:
-            j = r.json()
-            return j["result"]
-    except Exception:
-        return None
-
-
 def thread_collect_info(report, reportfile, package, ui, symptom_script=None,
                         ignore_uninstalled=False):
     '''Collect information about report.
@@ -112,7 +99,7 @@ def thread_collect_info(report, reportfile, package, ui, symptom_script=None,
             report['Symptom'] = os.path.splitext(os.path.basename(symptom_script))[0]
         except StopIteration:
             sys.exit(0)
-        except Exception as e:
+        except Exception:
             apport.error('symptom script %s crashed:', symptom_script)
             traceback.print_exc()
             sys.exit(0)
@@ -122,12 +109,20 @@ def thread_collect_info(report, reportfile, package, ui, symptom_script=None,
             package = apport.fileutils.find_file_package(report['ExecutablePath'])
         else:
             raise KeyError('called without a package, and report does not have ExecutablePath')
+
+    # check if the package name relates to an installed snap
+    snap = apport.fileutils.find_snap(package)
+    if snap:
+        report.add_snap_info(snap)
+
     try:
         report.add_package_info(package)
     except ValueError:
         # this happens if we are collecting information on an uninstalled
         # package
-        if not ignore_uninstalled:
+
+        # we found no package, but a snap, so lets continue
+        if not ignore_uninstalled and 'Snap' not in report:
             raise
     except SystemError as e:
         report['UnreportableReason'] = excstr(e)
@@ -140,17 +135,22 @@ def thread_collect_info(report, reportfile, package, ui, symptom_script=None,
         # the chance to set a third-party CrashDB.
         try:
             if 'CrashDB' not in report and 'APPORT_DISABLE_DISTRO_CHECK' not in os.environ:
-                if 'Package' not in report:
+                if 'Package' not in report and 'Snap' not in report:
                     report['UnreportableReason'] = _('This package does not seem to be installed correctly')
                 elif not apport.packaging.is_distro_package(report['Package'].split()[0]) and  \
                         not apport.packaging.is_native_origin_package(report['Package'].split()[0]):
                     # TRANS: %s is the name of the operating system
                     report['UnreportableReason'] = _(
-                        'This is not an official %s package. Please remove any third party package and try again.') % report['DistroRelease'].split()[0]
+                        'This does not seem to be an official %s package. \
+Please retry after updating the indexes of available packages, \
+if that does not work then remove related third party packages \
+and try again.') % report['DistroRelease'].split()[0]
         except ValueError:
             # this happens if we are collecting information on an uninstalled
             # package
-            if not ignore_uninstalled:
+
+            # we found no package, but a snap, so lets continue
+            if not ignore_uninstalled and 'Snap' not in report:
                 raise
 
     # add title
@@ -243,11 +243,11 @@ class UserInterface:
             logind_session = None
         else:
             reports = apport.fileutils.get_new_reports()
-            if PY3:
+            if not PY3:
+                logind_session = apport.Report.get_logind_session(os.getpid())
+            else:
                 proc_pid_fd = os.open('/proc/%s' % os.getpid(), os.O_RDONLY | os.O_PATH | os.O_DIRECTORY)
                 logind_session = apport.Report.get_logind_session(proc_pid_fd=proc_pid_fd)
-            else:
-                logind_session = apport.Report.get_logind_session(os.getpid())
 
         for f in reports:
             if not self.load_report(f):
@@ -259,6 +259,9 @@ class UserInterface:
             # and the session started before the report's "Date".
             if logind_session and '_LogindSession' in self.report and \
                'Date' in self.report:
+                # report.get_timestamp() can return None
+                if not self.report.get_timestamp():
+                    continue
                 if logind_session[0] != self.report['_LogindSession'] or \
                    logind_session[1] > self.report.get_timestamp():
                     continue
@@ -401,7 +404,17 @@ class UserInterface:
                                   _('You need to specify a PID. See --help for more information.'))
             return False
 
-        self.report.add_proc_info(pid)
+        try:
+            self.report.add_proc_info(pid)
+        except ValueError as e:
+            if str(e) == 'invalid process':
+                self.ui_error_message(_('Invalid PID'),
+                                      _('The specified process ID does not exist.'))
+                sys.exit(1)
+            elif str(e) == 'not accessible':
+                self.ui_error_message(_('Not your PID'),
+                                      _('The specified process ID does not belong to you.'))
+                sys.exit(1)
         self.report.add_package_info()
         path = self.report.get('ExecutablePath', '')
         self.cur_package = apport.fileutils.find_file_package(path)
@@ -463,7 +476,7 @@ class UserInterface:
                 proc_pid_fd = None
                 if PY3:
                     proc_pid_fd = os.open('/proc/%s' % self.options.pid, os.O_RDONLY | os.O_PATH | os.O_DIRECTORY)
-                    stat_file = os.open('stat', os.O_RDONLY | os.O_CLOEXEC, dir_fd=proc_pid_fd)
+                    stat_file = os.open('stat', os.O_RDONLY, dir_fd=proc_pid_fd)
                 else:
                     stat_file = '/proc/%s/stat' % self.options.pid
                 with io.open(stat_file) as f:
@@ -528,8 +541,13 @@ class UserInterface:
 
         if self.options.save:
             try:
-                with open(os.path.expanduser(self.options.save), 'wb') as f:
-                    self.report.write(f)
+                savefile = os.path.expanduser(self.options.save)
+                if savefile.endswith(".gz"):
+                    with gzip.open(savefile, 'wb') as f:
+                        self.report.write(f)
+                else:
+                    with open(os.path.expanduser(self.options.save), 'wb') as f:
+                        self.report.write(f)
             except (IOError, OSError) as e:
                 self.ui_error_message(_('Cannot create report'), excstr(e))
         else:
@@ -709,23 +727,36 @@ class UserInterface:
                 self.ui_error_message(_('Invalid problem report'), excstr(e))
             return True
         elif self.options.window:
-                self.ui_info_message('', _('After closing this message '
-                                           'please click on an application window to report a problem about it.'))
-                xprop = subprocess.Popen(['xprop', '_NET_WM_PID'],
-                                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                (out, err) = xprop.communicate()
-                if xprop.returncode == 0:
-                    try:
-                        self.options.pid = int(out.split()[-1])
-                    except ValueError:
-                        self.ui_error_message(_('Cannot create report'),
-                                              _('xprop failed to determine process ID of the window'))
-                        return True
-                    return self.run_report_bug()
-                else:
+            if os.getenv('XDG_SESSION_TYPE') == 'wayland':
+                self.ui_error_message(_('Cannot create report'),
+                                      _('The window option cannot be used on '
+                                        'Wayland.\n\nPlease find the window\'s '
+                                        'process ID and then run \'ubuntu-bug '
+                                        '<process ID>\'.'
+                                        '\n\nThe process ID can be found '
+                                        'by running the System Monitor application. '
+                                        'In the Processes tab, scroll until you '
+                                        'find the correct application. The process '
+                                        'ID is the number listed in the ID column.'))
+                return True
+
+            self.ui_info_message('', _('After closing this message '
+                                       'please click on an application window to report a problem about it.'))
+            xprop = subprocess.Popen(['xprop', '_NET_WM_PID'],
+                                     stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            (out, err) = xprop.communicate()
+            if xprop.returncode == 0:
+                try:
+                    self.options.pid = int(out.split()[-1])
+                except ValueError:
                     self.ui_error_message(_('Cannot create report'),
-                                          _('xprop failed to determine process ID of the window') + '\n\n' + err)
+                                          _('xprop failed to determine process ID of the window'))
                     return True
+                return self.run_report_bug()
+            else:
+                self.ui_error_message(_('Cannot create report'),
+                                      _('xprop failed to determine process ID of the window') + '\n\n' + err)
+                return True
         else:
             return self.run_crashes()
 
@@ -847,7 +878,7 @@ class UserInterface:
         elif '/' in self.args[0]:
             if self.args[0].startswith('/snap/bin'):
                 # see if the snap has the same name as the executable
-                snap = find_snap(self.args[0].split('/')[-1])
+                snap = apport.fileutils.find_snap(self.args[0].split('/')[-1])
                 if not snap:
                     optparser.error('%s is provided by a snap. No contact address has been provided; visit the forum at https://forum.snapcraft.io/ for help.' % self.args[0])
                 elif snap.get("contact", ""):
@@ -1100,7 +1131,7 @@ class UserInterface:
                                                     args=(self.report, self.report_file, self.cur_package,
                                                           hookui, symptom_script, ignore_uninstalled))
                 icthread.start()
-                while icthread.isAlive():
+                while icthread.is_alive():
                     self.ui_pulse_info_collection_progress()
                     if hookui:
                         try:
@@ -1123,16 +1154,6 @@ class UserInterface:
                         repr(e))
                     self.report['_MarkForUpload'] = 'False'
                 except ValueError:  # package does not exist
-                    snap = find_snap(self.cur_package)
-                    if not snap:
-                        pass
-                    elif snap.get("contact", ""):
-                        self.report['UnreportableReason'] = _('This report is about a snap published by %s. Contact them via %s for help.') % (snap["developer"], snap["contact"])
-                        self.report['_MarkForUpload'] = 'False'
-                    else:
-                        self.report['UnreportableReason'] = _('This report is about a snap published by %s. No contact address has been provided; visit the forum at https://forum.snapcraft.io/ for help.') % snap["developer"]
-                        self.report['_MarkForUpload'] = 'False'
-
                     if 'UnreportableReason' not in self.report:
                         self.report['UnreportableReason'] = _('This report is about a package that is not installed.')
                         self.report['_MarkForUpload'] = 'False'
@@ -1142,18 +1163,52 @@ class UserInterface:
                                                           'process this problem report:') + '\n\n' + str(e)
                     self.report['_MarkForUpload'] = 'False'
 
-            snap = find_snap(self.cur_package)
-            if snap and 'UnreportableReason' not in self.report and self.specified_a_pkg:
-                if snap.get("contact", ""):
-                    msg = _('You are about to report a bug against the deb package, but you also a have snap published by %s installed. You can contact them via %s for help. Do you want to continue with the bug report against the deb?') % (snap["developer"], snap["contact"])
+            # ask for target, if snap and deb package are installed
+            if 'Snap' in self.report and 'Package' in self.report:
+                if '(not installed)' in self.report['Package']:
+                    # choose snap automatically, if deb package is not installed
+                    res = [0]
+                else:
+                    res = self.ui_question_choice(_('You have two versions of this application installed, which one do you want to report a bug against?'), [
+                                                  _('%s snap') % self.report['Snap'],
+                                                  _('%s deb package') % self.report['Package']
+                                                  ], False)
+                # bug report is about the snap, clean deb package info
+                if res == [0]:
+                    del self.report['Package']
+                    if 'PackageArchitecture' in self.report:
+                        del self.report['PackageArchitecture']
+                    if 'SourcePackage' in self.report:
+                        del self.report['SourcePackage']
+                # bug report is about the deb package, clean snap info
+                elif res == [1]:
+                    del self.report['Snap']
+                    if 'SnapSource' in self.report:
+                        del self.report['SnapSource']
+                    if 'SnapTags' in self.report:
+                        del self.report['SnapTags']
                 else:
-                    msg = _('You are about to report a bug against the deb package, but you also a have snap published by %s installed. For the snap, no contact address has been provided; visit the forum at https://forum.snapcraft.io/ for help. Do you want to continue with the bug report against the deb?') % snap["developer"]
-
-                if not self.ui_question_yesno(msg):
                     self.ui_stop_info_collection_progress()
                     sys.exit(0)
                     return
 
+            # append snap tags, if this report is about the snap
+            if 'Snap' in self.report and 'SnapTags' in self.report:
+                current_tags = self.report.get('Tags', '')
+                if current_tags:
+                    current_tags += ' '
+                self.report['Tags'] = current_tags + self.report['SnapTags']
+                del self.report['SnapTags']
+
+            # show a hint if we cannot auto report a snap bug via 'SnapSource'
+            if 'Snap' in self.report and 'SnapSource' not in self.report and 'UnreportableReason' not in self.report and self.specified_a_pkg:
+                snap = apport.fileutils.find_snap(self.cur_package)
+                if snap.get("contact", ""):
+                    self.report['UnreportableReason'] = _('%s is provided by a snap published by %s. Contact them via %s for help.') % (snap["name"], snap["developer"], snap["contact"])
+                else:
+                    self.report['UnreportableReason'] = _('%s is provided by a snap published by %s. No contact address has been provided; visit the forum at https://forum.snapcraft.io/ for help.') % (snap["name"], snap["developer"])
+                self.report['_MarkForUpload'] = 'False'
+
             if 'UnreportableReason' in self.report or not self.check_report_crashdb():
                 self.ui_stop_info_collection_progress()
                 if on_finished:
@@ -1165,7 +1220,7 @@ class UserInterface:
                 bpthread = apport.REThread.REThread(target=self.report.search_bug_patterns,
                                                     args=(self.crashdb.get_bugpattern_baseurl(),))
                 bpthread.start()
-                while bpthread.isAlive():
+                while bpthread.is_alive():
                     self.ui_pulse_info_collection_progress()
                     try:
                         bpthread.join(0.1)
@@ -1186,7 +1241,7 @@ class UserInterface:
                 known_thread = apport.REThread.REThread(target=self.crashdb.known,
                                                         args=(self.report,))
                 known_thread.start()
-                while known_thread.isAlive():
+                while known_thread.is_alive():
                     self.ui_pulse_info_collection_progress()
                     try:
                         known_thread.join(0.1)
@@ -1204,7 +1259,7 @@ class UserInterface:
             # might damage the stack trace
             anonymize_thread = apport.REThread.REThread(target=self.report.anonymize)
             anonymize_thread.start()
-            while anonymize_thread.isAlive():
+            while anonymize_thread.is_alive():
                 self.ui_pulse_info_collection_progress()
                 try:
                     anonymize_thread.join(0.1)
@@ -1216,9 +1271,9 @@ class UserInterface:
 
             # check that we were able to determine package names
             if 'UnreportableReason' not in self.report:
-                if (('SourcePackage' not in self.report and 'Dependencies' not in self.report) or
+                if ((('SourcePackage' not in self.report and 'Dependencies' not in self.report) or
                     (not self.report.get('ProblemType', '').startswith('Kernel') and
-                     'Package' not in self.report)):
+                     'Package' not in self.report)) and ('SnapSource' not in self.report and 'Snap' not in self.report)):
                     self.ui_error_message(_('Invalid problem report'),
                                           _('Could not determine the package or source package name.'))
                     # TODO This is not called consistently, is it really needed?
@@ -1283,7 +1338,7 @@ class UserInterface:
         try:
             try:
                 subprocess.call(sudo_prefix + ['xdg-open', url])
-            except OSError as e:
+            except OSError:
                 # fall back to webbrowser
                 webbrowser.open(url, new=True, autoraise=True)
                 sys.exit(0)
@@ -1331,7 +1386,7 @@ class UserInterface:
         upthread = apport.REThread.REThread(target=self.crashdb.upload,
                                             args=(self.report, progress_callback))
         upthread.start()
-        while upthread.isAlive():
+        while upthread.is_alive():
             self.ui_set_upload_progress(__upload_progress)
             try:
                 upthread.join(0.1)
